"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var createSubscribable_1 = require("./createSubscribable");
/**
 * An event that fires its listeners in a synchronous fashion.
 */
var Event = /** @class */ (function () {
    /**
     * Create a new event.
     *
     * @param parent
     *   the parent that will be passed to listener as their `this`
     */
    function Event(parent) {
        this.parent = parent;
        this.subscribable = createSubscribable_1.createSubscribable(this.subscribe0.bind(this), this.unsubscribe0.bind(this));
    }
    /**
     * Emit this event. This will invoke all of the listeners with the passed
     * arguments.
     *
     * @param args
     */
    Event.prototype.emit = function () {
        var e_1, _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (Array.isArray(this.registeredListeners)) {
            try {
                /*
                 * Array is present, iterate over array and invoke all of the
                 * listeners.
                 */
                for (var _b = __values(this.registeredListeners), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var listener = _c.value;
                    listener.apply(this.parent, args);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        else if (this.registeredListeners) {
            /*
             * Single listener is present, simply invoke the listener.
             */
            this.registeredListeners.apply(this.parent, args);
        }
    };
    /**
     * Subscribe to this event using the given listener. The listener will
     * be invoked any time the event is emitted. The returned handle can be
     * used to unsubscribe.
     *
     * @param listener
     */
    Event.prototype.subscribe0 = function (listener) {
        if (Array.isArray(this.registeredListeners)) {
            // Listeners is already an array, create a copy with the new listener appended
            var idx = this.registeredListeners.indexOf(listener);
            if (idx >= 0) {
                // If the listener is already in the array, skip registering
                return;
            }
            this.registeredListeners = __spread(this.registeredListeners, [listener]);
        }
        else if (this.registeredListeners) {
            if (this.registeredListeners === listener) {
                // This is the active listener, skip registering
                return;
            }
            this.registeredListeners = [this.registeredListeners, listener];
        }
        else {
            this.registeredListeners = listener;
        }
        if (this.monitor) {
            // Trigger the monitor if available
            this.monitor(this);
        }
        return;
    };
    /**
     * Unsubscribe a listener from this handler. The specified listener will
     * no longer be invoked when the event is emitted.
     *
     * @param listener
     */
    Event.prototype.unsubscribe0 = function (listener) {
        if (Array.isArray(this.registeredListeners)) {
            /*
             * Array has been allocated, find the index of the listener and
             * then remove it from the array.
             */
            var idx = this.registeredListeners.indexOf(listener);
            if (idx < 0)
                return false;
            // Copy-on-write for deletions
            var listeners = __spread(this.registeredListeners);
            listeners.splice(idx);
            this.registeredListeners = listener;
            /*
             * If the array is empty, remove it. Otherwise at this point the
             * array has already been allocated so keep the array in case a
             * subscription happens again.
             */
            if (this.registeredListeners.length === 0) {
                this.registeredListeners = undefined;
            }
            if (this.monitor) {
                // Trigger the monitor if available
                this.monitor(this);
            }
            return true;
        }
        else if (this.registeredListeners === listener) {
            /*
             * Single listener is present and its the current match. Reset
             * listeners property.
             */
            this.registeredListeners = undefined;
            if (this.monitor) {
                // Trigger the monitor if available
                this.monitor(this);
            }
            return true;
        }
        // Listener is not active
        return false;
    };
    /**
     * Subscribe to this event using the given listener. The listener will
     * be invoked any time the event is emitted. The returned handle can be
     * used to unsubscribe.
     *
     * @param listener
     */
    Event.prototype.subscribe = function (listener) {
        return this.subscribable.subscribe(listener);
    };
    /**
     * Unsubscribe a listener from this handler. The specified listener will
     * no longer be invoked when the event is emitted.
     *
     * @param listener
     */
    Event.prototype.unsubscribe = function (listener) {
        return this.subscribable.unsubscribe(listener);
    };
    /**
     * Get a promise that will resolve the first time this event is fired
     * after this call.
     */
    Event.prototype.once = function () {
        return this.subscribable.once();
    };
    /**
     * Create a subscribable that will apply the specified filter to any
     * listeners added.
     *
     * @param filter
     */
    Event.prototype.filter = function (filter) {
        return this.subscribable.filter(filter);
    };
    /**
     * Create a Subscribable that changes the this argument used for listeners.
     *
     * @param newThis
     */
    Event.prototype.withThis = function (newThis) {
        return this.subscribable.withThis(newThis);
    };
    Object.defineProperty(Event.prototype, "hasListeners", {
        /**
         * Get if there are any listeners available.
         */
        get: function () {
            return this.registeredListeners !== undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Event.prototype, "listeners", {
        /**
         * Get a copy of the listeners as an array.
         */
        get: function () {
            if (Array.isArray(this.registeredListeners)) {
                return this.registeredListeners.slice(0);
            }
            else if (this.registeredListeners) {
                return [this.registeredListeners];
            }
            else {
                return [];
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Clear all listeners for this event.
     */
    Event.prototype.clear = function () {
        this.registeredListeners = undefined;
        if (this.monitor) {
            // Trigger the monitor if available
            this.monitor(this);
        }
    };
    /**
     * Monitor for changes to listeners. Only a single monitor is supported at
     * a single time. This is intended to be used to react to if listeners are
     * currently registered. This can be used for things such as only listening
     * to events from other objects when this event is active.
     *
     * @param monitor
     */
    Event.prototype.monitorListeners = function (monitor) {
        if (this.monitor) {
            throw new Error('A monitor is already registered, call removeMonitor before registering a new monitor');
        }
        this.monitor = monitor;
    };
    /**
     * Stop monitoring for listener changes.
     */
    Event.prototype.removeMonitor = function () {
        this.monitor = undefined;
    };
    return Event;
}());
exports.Event = Event;
//# sourceMappingURL=Event.js.map