"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Create a Subscribable given a subscribe, unsubscribe and a once function.
 *
 * @param subscribe
 * @param unsubscribe
 */
function createSubscribable(subscribe, unsubscribe) {
    var subscribable = function (listener) {
        subscribe(listener);
        // Return a handle that can be used to unsubscribe
        return {
            unsubscribe: function () {
                unsubscribe(listener);
            }
        };
    };
    subscribable.subscribe = subscribable;
    subscribable.unsubscribe = unsubscribe;
    subscribable.once = function () { return new Promise(function (resolve, reject) {
        var listener = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            unsubscribe(listener);
            resolve(args);
        };
        subscribe(listener);
    }); };
    subscribable.filter = function (filter) { return createFilteredSubscribable(subscribe, unsubscribe, filter); };
    subscribable.withThis = function (newThis) { return createNewThisSubscribable(subscribe, unsubscribe, newThis); };
    return subscribable;
}
exports.createSubscribable = createSubscribable;
/**
 * Create a Subscribable that is filtered via the specified function.
 *
 * @param subscribe
 * @param unsubscribe
 * @param filterToApply
 */
function createFilteredSubscribable(subscribe, unsubscribe, filterToApply) {
    // Map used to keep track of the filtered listener of an added listener
    var listenerMapping = new Map();
    return createSubscribable(function (listener) {
        var actualListener = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (filterToApply.apply(this, args)) {
                listener.call.apply(listener, __spread([this], args));
            }
        };
        listenerMapping.set(listener, actualListener);
        subscribe(actualListener);
    }, function (listener) {
        var actual = listenerMapping.get(listener);
        if (actual) {
            listenerMapping.delete(listener);
            return unsubscribe(actual);
        }
        else {
            return false;
        }
    });
}
/**
 * Create a Subscribable that changes what this is used for listeners.
 *
 * @param subscribe
 * @param unsubscribe
 * @param filterToApply
 */
function createNewThisSubscribable(subscribe, unsubscribe, newThis) {
    // Map used to keep track of the modified listeners
    var listenerMapping = new Map();
    return createSubscribable(function (listener) {
        var actualListener = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            listener.call.apply(listener, __spread([newThis], args));
        };
        listenerMapping.set(listener, actualListener);
        subscribe(actualListener);
    }, function (listener) {
        var actual = listenerMapping.get(listener);
        if (actual) {
            listenerMapping.delete(listener);
            return unsubscribe(actual);
        }
        else {
            return false;
        }
    });
}
//# sourceMappingURL=createSubscribable.js.map